@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,11 +19,11 @@
 #include <ofono/modem.h>
 #include <ofono/call-forwarding.h>
 
+#include <drivers/atmodem/atutil.h>
+
 #include "gatchat.h"
 #include "gatresult.h"
 
-#include "atmodem.h"
-
 static const char *none_prefix[] = { NULL };
 static const char *ccfc_prefix[] = { "+CCFC:", NULL };
 
@@ -264,7 +250,6 @@ static void at_ccfc_remove(struct ofono_call_forwarding *cf)
 }
 
 static const struct ofono_call_forwarding_driver driver = {
-	.name		= "atmodem",
 	.probe		= at_ccfc_probe,
 	.remove		= at_ccfc_remove,
 	.registration	= at_ccfc_registration,
@@ -274,12 +259,4 @@ static const struct ofono_call_forwarding_driver driver = {
 	.erasure	= at_ccfc_erasure
 };
 
-void at_call_forwarding_init(void)
-{
-	ofono_call_forwarding_driver_register(&driver);
-}
-
-void at_call_forwarding_exit(void)
-{
-	ofono_call_forwarding_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(call_forwarding, atmodem, &driver)
