@@ -353,6 +353,13 @@ static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	op.status = 2;
 	op.tech = tech;
 
+	/*
+	 * SIMCom modems shows the current tech via the +COPS
+	 * not like other modems where it's shown via +CREG
+	 */
+	if (nd->vendor == OFONO_VENDOR_SIMCOM_A76XX)
+		nd->tech = tech;
+
 	DBG("cops_cb: %s, %s %s %d", name, nd->mcc, nd->mnc, tech);
 
 	cb(&error, &op, cbd->data);
@@ -676,7 +683,6 @@ static void option_osigq_notify(GAtResult *result, gpointer user_data)
 
 static void ifx_xhomezr_notify(GAtResult *result, gpointer user_data)
 {
-	//struct ofono_netreg *netreg = user_data;
 	const char *label;
 	GAtResultIter iter;
 
@@ -741,7 +747,6 @@ static void ifx_xreg_notify(GAtResult *result, gpointer user_data)
 
 static void ifx_xciev_notify(GAtResult *result, gpointer user_data)
 {
-	//struct ofono_netreg *netreg = user_data;
 	int ind;
 	GAtResultIter iter;
 
