@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2009-2011  Nokia Corporation and/or its subsidiary(-ies)
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2009-2011  Nokia Corporation and/or its subsidiary(-ies).
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,7 +23,6 @@
 #include <ofono/modem.h>
 #include <ofono/call-settings.h>
 
-#include "isimodem.h"
 #include "isiutil.h"
 #include "ss.h"
 #include "debug.h"
@@ -405,24 +390,13 @@ static void isi_call_settings_remove(struct ofono_call_settings *cs)
 }
 
 static const struct ofono_call_settings_driver driver = {
-	.name			= "isimodem",
 	.probe			= isi_call_settings_probe,
 	.remove			= isi_call_settings_remove,
 	.clip_query		= isi_clip_query,
 	.colp_query		= isi_colp_query,
 	.colr_query		= isi_colr_query,
-	.clir_query		= NULL,
-	.clir_set		= NULL,
 	.cw_query		= isi_cw_query,
 	.cw_set			= isi_cw_set
 };
 
-void isi_call_settings_init(void)
-{
-	ofono_call_settings_driver_register(&driver);
-}
-
-void isi_call_settings_exit(void)
-{
-	ofono_call_settings_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(call_settings, isimodem, &driver)
