@@ -1,23 +1,9 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2011  ST-Ericsson AB
+ * Copyright (C) 2009-2010  Nokia Corporation and/or its subsidiary(-ies)
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2011  ST-Ericsson AB.
- *  Copyright (C) 2009-2010  Nokia Corporation and/or its subsidiary(-ies).
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,7 +28,6 @@
 #include <ofono/sms.h>
 
 #include "smsutil.h"
-#include "isimodem.h"
 #include "isiutil.h"
 #include "sms.h"
 #include "sim.h"
@@ -1121,7 +1106,6 @@ static void isi_sms_remove(struct ofono_sms *sms)
 }
 
 static const struct ofono_sms_driver driver = {
-	.name			= "isimodem",
 	.probe			= isi_sms_probe,
 	.remove			= isi_sms_remove,
 	.sca_query		= isi_sca_query,
@@ -1131,12 +1115,4 @@ static const struct ofono_sms_driver driver = {
 	.bearer_set		= isi_bearer_set,
 };
 
-void isi_sms_init(void)
-{
-	ofono_sms_driver_register(&driver);
-}
-
-void isi_sms_exit(void)
-{
-	ofono_sms_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(sms, isimodem, &driver)
