@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2017  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2017  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -496,7 +482,6 @@ static void mbim_sms_remove(struct ofono_sms *sms)
 }
 
 static const struct ofono_sms_driver driver = {
-	.name		= "mbim",
 	.probe		= mbim_sms_probe,
 	.remove		= mbim_sms_remove,
 	.sca_query	= mbim_sca_query,
@@ -504,12 +489,4 @@ static const struct ofono_sms_driver driver = {
 	.submit		= mbim_submit,
 };
 
-void mbim_sms_init(void)
-{
-	ofono_sms_driver_register(&driver);
-}
-
-void mbim_sms_exit(void)
-{
-	ofono_sms_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(sms, mbim, &driver)
