@@ -1,24 +1,9 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2014 Jolla Lt
+ * Copyright (C) 2014  Canonical Lt
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2014 Jolla Ltd
- *  Contact: Miia Leinonen
- *  Copyright (C) 2014  Canonical Ltd
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #include <errno.h>
@@ -31,8 +16,7 @@
 #include "common.h"
 
 #include "gril.h"
-
-#include "rilmodem.h"
+#include "rilutil.h"
 
 struct barring_data {
 	GRil *ril;
@@ -271,7 +255,6 @@ static void ril_call_barring_remove(struct ofono_call_barring *cb)
 }
 
 static const struct ofono_call_barring_driver driver = {
-	.name			= "rilmodem",
 	.probe			= ril_call_barring_probe,
 	.remove			= ril_call_barring_remove,
 	.query			= ril_call_barring_query,
@@ -279,12 +262,4 @@ static const struct ofono_call_barring_driver driver = {
 	.set_passwd		= ril_call_barring_set_passwd
 };
 
-void ril_call_barring_init(void)
-{
-	ofono_call_barring_driver_register(&driver);
-}
-
-void ril_call_barring_exit(void)
-{
-	ofono_call_barring_driver_unregister(&driver);
-}
+OFONO_ATOM_DRIVER_BUILTIN(call_barring, rilmodem, &driver)
