@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifndef __OFONO_GPRS_H
@@ -26,6 +12,8 @@
 extern "C" {
 #endif
 
+#include <stdarg.h>
+
 #include <ofono/types.h>
 
 struct ofono_gprs;
@@ -37,9 +25,11 @@ typedef void (*ofono_gprs_status_cb_t)(const struct ofono_error *error,
 typedef void (*ofono_gprs_cb_t)(const struct ofono_error *error, void *data);
 
 struct ofono_gprs_driver {
-	const char *name;
+	unsigned int flags;
 	int (*probe)(struct ofono_gprs *gprs, unsigned int vendor,
 			void *data);
+	int (*probev)(struct ofono_gprs *gprs, unsigned int vendor,
+			va_list args);
 	void (*remove)(struct ofono_gprs *gprs);
 	void (*set_attached)(struct ofono_gprs *gprs, int attached,
 				ofono_gprs_cb_t cb, void *data);
@@ -65,12 +55,9 @@ void ofono_gprs_bearer_notify(struct ofono_gprs *gprs, int bearer);
 
 struct ofono_modem *ofono_gprs_get_modem(struct ofono_gprs *gprs);
 
-int ofono_gprs_driver_register(const struct ofono_gprs_driver *d);
-void ofono_gprs_driver_unregister(const struct ofono_gprs_driver *d);
-
 struct ofono_gprs *ofono_gprs_create(struct ofono_modem *modem,
 					unsigned int vendor, const char *driver,
-					void *data);
+					...);
 void ofono_gprs_register(struct ofono_gprs *gprs);
 void ofono_gprs_remove(struct ofono_gprs *gprs);
 
