@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifndef __OFONO_VOICECALL_H
@@ -26,6 +12,8 @@
 extern "C" {
 #endif
 
+#include <stdarg.h>
+
 #include <ofono/types.h>
 
 struct ofono_modem;
@@ -41,9 +29,11 @@ typedef void (*ofono_voicecall_cb_t)(const struct ofono_error *error,
  * not support vendor extensions for call progress indication.
  */
 struct ofono_voicecall_driver {
-	const char *name;
+	unsigned int flags;
 	int (*probe)(struct ofono_voicecall *vc, unsigned int vendor,
 			void *data);
+	int (*probev)(struct ofono_voicecall *vc, unsigned int vendor,
+			va_list args);
 	void (*remove)(struct ofono_voicecall *vc);
 
 	/* According to 22.030 the dial is expected to do the following:
@@ -162,12 +152,9 @@ void ofono_voicecall_mpty_hint(struct ofono_voicecall *vc, unsigned int ids);
 
 struct ofono_modem *ofono_voicecall_get_modem(struct ofono_voicecall *vc);
 
-int ofono_voicecall_driver_register(const struct ofono_voicecall_driver *d);
-void ofono_voicecall_driver_unregister(const struct ofono_voicecall_driver *d);
-
 struct ofono_voicecall *ofono_voicecall_create(struct ofono_modem *modem,
 					unsigned int vendor,
-					const char *driver, void *data);
+					const char *driver, ...);
 
 void ofono_voicecall_register(struct ofono_voicecall *vc);
 void ofono_voicecall_remove(struct ofono_voicecall *vc);
