@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -285,35 +271,12 @@ static void emrdy_query(gboolean ok, GAtResult *result, gpointer user_data)
 					cfun_query, modem, NULL);
 }
 
-static GAtChat *create_port(const char *device)
+static GAtChat *open_device(struct ofono_modem *modem, const char *key,
+				char *debug)
 {
-	GAtSyntax *syntax;
-	GIOChannel *channel;
-	GAtChat *chat;
-	GHashTable *options;
-
-	options = g_hash_table_new(g_str_hash, g_str_equal);
-	if (options == NULL)
-		return NULL;
-
-	g_hash_table_insert(options, "Baud", "115200");
-
-	channel = g_at_tty_open(device, options);
-
-	g_hash_table_destroy(options);
-
-	if (channel == NULL)
-		return NULL;
-
-	syntax = g_at_syntax_new_gsm_permissive();
-	chat = g_at_chat_new(channel, syntax);
-	g_at_syntax_unref(syntax);
-	g_io_channel_unref(channel);
-
-	if (chat == NULL)
-		return NULL;
-
-	return chat;
+	return at_util_open_device(modem, key, mbm_debug, debug,
+					"Baud", "115200",
+					NULL);
 }
 
 static int mbm_enable(struct ofono_modem *modem)
@@ -332,14 +295,11 @@ static int mbm_enable(struct ofono_modem *modem)
 	if (modem_dev == NULL || data_dev == NULL)
 		return -EINVAL;
 
-	data->modem_port = create_port(modem_dev);
+	data->modem_port = open_device(modem, "ModemDevice", "Modem: ");
 	if (data->modem_port == NULL)
 		return -EIO;
 
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->modem_port, mbm_debug, "Modem: ");
-
-	data->data_port = create_port(data_dev);
+	data->data_port = open_device(modem, "DataDevice", "Data: ");
 	if (data->data_port == NULL) {
 		g_at_chat_unref(data->modem_port);
 		data->modem_port = NULL;
@@ -347,9 +307,6 @@ static int mbm_enable(struct ofono_modem *modem)
 		return -EIO;
 	}
 
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->data_port, mbm_debug, "Data: ");
-
 	g_at_chat_register(data->modem_port, "*EMRDY:", emrdy_notifier,
 					FALSE, modem, NULL);
 
@@ -506,7 +463,6 @@ static void mbm_post_online(struct ofono_modem *modem)
 }
 
 static struct ofono_modem_driver mbm_driver = {
-	.name		= "mbm",
 	.probe		= mbm_probe,
 	.remove		= mbm_remove,
 	.enable		= mbm_enable,
@@ -517,15 +473,4 @@ static struct ofono_modem_driver mbm_driver = {
 	.post_online	= mbm_post_online,
 };
 
-static int mbm_init(void)
-{
-	return ofono_modem_driver_register(&mbm_driver);
-}
-
-static void mbm_exit(void)
-{
-	ofono_modem_driver_unregister(&mbm_driver);
-}
-
-OFONO_PLUGIN_DEFINE(mbm, "Ericsson MBM modem driver", VERSION,
-			OFONO_PLUGIN_PRIORITY_DEFAULT, mbm_init, mbm_exit)
+OFONO_MODEM_DRIVER_BUILTIN(mbm, &mbm_driver)
