@@ -1045,7 +1045,8 @@ static gboolean setup_quectel_usb(struct modem_info *modem)
 			mdm = info->devnode;
 			if (aux != NULL)
 				break;
-		} else if (g_strcmp0(info->interface, "255/255/255") == 0) {
+		} else if (g_strcmp0(info->interface, "255/255/255") == 0 ||
+				g_strcmp0(info->interface, "255/0/0") == 0) {
 			if (g_strcmp0(info->number, "02") == 0)
 				aux = info->devnode;
 			else if (g_strcmp0(info->number, "03") == 0)
@@ -1107,19 +1108,6 @@ static gboolean setup_quectel(struct modem_info *modem)
 		return FALSE;
 }
 
-static gboolean is_premultiplexed(const struct device_info *net)
-{
-	struct udev_device *parent = udev_device_get_parent(net->udev_device);
-
-	if (!parent)
-		return FALSE;
-
-	if (g_strcmp0(udev_device_get_subsystem(parent), "net") == 0)
-		return TRUE;
-
-	return FALSE;
-}
-
 static gboolean setup_quectelqmi(struct modem_info *modem)
 {
 	const struct device_info *net = NULL;
@@ -1127,14 +1115,23 @@ static gboolean setup_quectelqmi(struct modem_info *modem)
 	const char *gps = NULL;
 	const char *aux = NULL;
 	GSList *list;
-	const char *premux_interfaces[8];
-	int n_premux = 0;
-	const char *qmap_size;
-
-	memset(premux_interfaces, 0, sizeof(premux_interfaces));
 
 	DBG("%s", modem->syspath);
 
+	/*
+	 * The Quectel Wireless Solutions Co., Ltd. (2c7c) BG96
+	 * CAT-M1/NB-IoT modem (0296) has a firmware issue where it can
+	 * lock up and hang (not responding to subsequent commands) due to
+	 * high QMI service request arrival rates. If the vendor and model
+	 * match those, then rate limit QMI service requests to no more
+	 * than one every 2,000 us.
+	 */
+	if (l_streq0(modem->vendor, "2c7c")) {
+		if (l_streq0(modem->model, "0296"))
+			ofono_modem_set_integer(modem->modem,
+				"RequestThrottleTimeUs", 2000);
+	}
+
 	for (list = modem->devices; list; list = g_slist_next(list)) {
 		const struct device_info *info = list->data;
 		const char *subsystem =
@@ -1143,22 +1140,12 @@ static gboolean setup_quectelqmi(struct modem_info *modem)
 		DBG("%s %s %s %s %s", info->devnode, info->interface,
 				info->number, info->label, subsystem);
 
-		if (g_strcmp0(info->interface, "255/255/255") == 0) {
-			if (g_strcmp0(subsystem, "usbmisc") == 0) {
+		if (g_strcmp0(info->interface, "255/255/255") == 0 &&
+				g_strcmp0(info->number, "04") == 0) {
+			if (g_strcmp0(subsystem, "net") == 0)
+				net = info;
+			else if (g_strcmp0(subsystem, "usbmisc") == 0)
 				qmi = info;
-				continue;
-			}
-
-			if (g_strcmp0(subsystem, "net"))
-				continue;
-
-			if (is_premultiplexed(info)) {
-				premux_interfaces[n_premux] = info->devnode;
-				n_premux += 1;
-				continue;
-			}
-
-			net = info;
 		} else if (g_strcmp0(info->interface, "255/0/0") == 0 &&
 				g_strcmp0(info->number, "01") == 0) {
 			gps = info->devnode;
@@ -1176,44 +1163,12 @@ static gboolean setup_quectelqmi(struct modem_info *modem)
 	if (setup_qmi_qmux(modem, qmi, net) < 0)
 		return FALSE;
 
-	qmap_size = udev_device_get_sysattr_value(net->udev_device,
-							"qmap_size");
-	if (qmap_size) {
-		uint32_t max_aggregation_size;
-
-		if (l_safe_atou32(qmap_size, &max_aggregation_size) == 0)
-			ofono_modem_set_integer(modem->modem,
-						"MaxAggregationSize",
-						max_aggregation_size);
-	}
-
 	if (gps)
 		ofono_modem_set_string(modem->modem, "GPS", gps);
 
 	if (aux)
 		ofono_modem_set_string(modem->modem, "Aux", aux);
 
-	if (n_premux) {
-		char buf[256];
-		int i;
-
-		ofono_modem_set_integer(modem->modem,
-					"NumPremuxInterfaces", n_premux);
-		for (i = 0; i < n_premux; i++) {
-			const char *device = premux_interfaces[i];
-			int len = strlen(device);
-
-			if (!len)
-				continue;
-
-			sprintf(buf, "PremuxInterface%d", i + 1);
-			ofono_modem_set_string(modem->modem, buf, device);
-			sprintf(buf, "PremuxInterface%dMuxId", i + 1);
-			ofono_modem_set_integer(modem->modem, buf,
-						0x80 + device[len - 1] - '0');
-		}
-	}
-
 	return TRUE;
 }
 
@@ -2145,6 +2100,7 @@ static struct {
 	{ "samsung",	"option",	"04e8", "6889"	},
 	{ "samsung",	"kalmia"			},
 	{ "quectel",	"option",	"05c6", "9090"	},
+	{ "quectel",	"option",	"2c7c", "6007"	},
 	{ "quectelqmi",	"qmi_wwan",	"2c7c", "0121"	},
 	{ "quectelqmi",	"qcserial",	"2c7c", "0121"	},
 	{ "quectelqmi",	"qmi_wwan",	"2c7c", "0125"	},
@@ -2158,7 +2114,6 @@ static struct {
 	{ "quectelqmi", "qmi_wwan",	"2c7c", "0800"	},
 	{ "quectelqmi", "qcserial",	"2c7c", "0800"	},
 	{ "quectelqmi", "option",	"2c7c", "0800"	},
-	{ "quectelqmi", "qmi_wwan_q",	"2c7c", "0452"	},
 	{ "ublox",	"cdc_acm",	"1546", "1010"	},
 	{ "ublox",	"cdc_ncm",	"1546", "1010"	},
 	{ "ublox",	"cdc_acm",	"1546", "1102"	},
@@ -2453,6 +2408,10 @@ static void check_net_device(struct udev_device *device)
 	}
 
 	name = udev_device_get_sysname(device);
+	if (l_str_has_prefix(name, "phonet")) {
+		add_serial_device(device);
+		return;
+	}
 	if (!l_str_has_prefix(name, "rmnet_"))
 		return;
 
