@@ -1,23 +1,9 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
+ * Copyright (C) 2011  BMW Car IT GmbH
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *  Copyright (C) 2011  BMW Car IT GmbH. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,8 +27,6 @@
 #include "common.h"
 #include "hfp.h"
 
-static GSList *g_drivers = NULL;
-
 #define HANDSFREE_FLAG_CACHED 0x1
 
 struct ofono_handsfree {
@@ -174,7 +158,7 @@ void ofono_handsfree_battchg_notify(struct ofono_handsfree *hf,
 					unsigned char level)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = __ofono_atom_get_path(hf->atom);
+	const char *path;
 
 	if (hf == NULL)
 		return;
@@ -187,6 +171,7 @@ void ofono_handsfree_battchg_notify(struct ofono_handsfree *hf,
 	if (__ofono_atom_get_registered(hf->atom) == FALSE)
 		return;
 
+	path = __ofono_atom_get_path(hf->atom);
 	ofono_dbus_signal_property_changed(conn, path,
 					OFONO_HANDSFREE_INTERFACE,
 					"BatteryChargeLevel", DBUS_TYPE_BYTE,
@@ -295,7 +280,7 @@ static void hf_cnum_callback(const struct ofono_error *error, int total,
 
 		subscriber_number->type = numbers[num].type;
 		strncpy(subscriber_number->number, numbers[num].number,
-					OFONO_MAX_PHONE_NUMBER_LENGTH + 1);
+					OFONO_MAX_PHONE_NUMBER_LENGTH);
 
 		hf->subscriber_numbers = g_slist_prepend(hf->subscriber_numbers,
 					subscriber_number);
@@ -503,7 +488,6 @@ static DBusMessage *handsfree_set_property(DBusConnection *conn,
 	dbus_message_iter_get_basic(&var, &enabled);
 
 	if (g_str_equal(name, "VoiceRecognition") == TRUE) {
-
 		if (!hf->driver->voice_recognition)
 			return __ofono_error_not_implemented(msg);
 
@@ -514,7 +498,6 @@ static DBusMessage *handsfree_set_property(DBusConnection *conn,
 		hf->pending = dbus_message_ref(msg);
 		hf->driver->voice_recognition(hf, enabled, voicerec_set_cb, hf);
 	} else if (g_str_equal(name, "EchoCancelingNoiseReduction") == TRUE) {
-
 		if (!(hf->ag_features & HFP_AG_FEATURE_ECNR))
 			return __ofono_error_not_supported(msg);
 
@@ -638,41 +621,9 @@ static void handsfree_remove(struct ofono_atom *atom)
 	g_free(hf);
 }
 
-struct ofono_handsfree *ofono_handsfree_create(struct ofono_modem *modem,
-					unsigned int vendor,
-					const char *driver,
-					void *data)
-{
-	struct ofono_handsfree *hf;
-	GSList *l;
-
-	if (driver == NULL)
-		return NULL;
-
-	hf = g_try_new0(struct ofono_handsfree, 1);
-	if (hf == NULL)
-		return NULL;
-
-	hf->atom = __ofono_modem_add_atom(modem,
-					OFONO_ATOM_TYPE_HANDSFREE,
-					handsfree_remove, hf);
-	hf->nrec = TRUE;
-
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_handsfree_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(hf, vendor, data) < 0)
-			continue;
-
-		hf->driver = drv;
-		break;
-	}
-
-	return hf;
-}
+OFONO_DEFINE_ATOM_CREATE(handsfree, OFONO_ATOM_TYPE_HANDSFREE, {
+	atom->nrec = TRUE;
+})
 
 static void handsfree_unregister(struct ofono_atom *atom)
 {
@@ -715,26 +666,6 @@ void ofono_handsfree_register(struct ofono_handsfree *hf)
 	__ofono_atom_register(hf->atom, handsfree_unregister);
 }
 
-int ofono_handsfree_driver_register(const struct ofono_handsfree_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_handsfree_driver_unregister(
-				const struct ofono_handsfree_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 void ofono_handsfree_remove(struct ofono_handsfree *hf)
 {
 	__ofono_atom_free(hf->atom);
