@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,13 +14,14 @@
 #include <unistd.h>
 #include <stdarg.h>
 #include <stdlib.h>
-#include <string.h>
 #include <syslog.h>
-#ifdef __GLIBC__
+#ifdef HAVE_BACKTRACE
 #include <execinfo.h>
 #endif
 #include <dlfcn.h>
 
+#include <ell/ell.h>
+
 #include "ofono.h"
 
 static const char *program_exec;
@@ -115,7 +102,7 @@ void ofono_debug(const char *format, ...)
 	va_end(ap);
 }
 
-#ifdef __GLIBC__
+#ifdef HAVE_BACKTRACE
 static void print_backtrace(unsigned int offset)
 {
 	void *frames[99];
@@ -171,11 +158,13 @@ static void print_backtrace(unsigned int offset)
 
 	for (i = offset; i < n_ptrs - 1; i++) {
 		Dl_info info;
+		const char *fname = "???";
 		char addr[20], buf[PATH_MAX * 2];
 		int len, written;
 		char *ptr, *pos;
 
-		dladdr(frames[i], &info);
+		if (dladdr(frames[i], &info))
+			fname = info.dli_fname;
 
 		len = snprintf(addr, sizeof(addr), "%p\n", frames[i]);
 		if (len < 0)
@@ -199,7 +188,7 @@ static void print_backtrace(unsigned int offset)
 
 		if (strcmp(buf, "??") == 0) {
 			ofono_error("#%-2u %p in %s", i - offset,
-						frames[i], info.dli_fname);
+						frames[i], fname);
 			continue;
 		}
 
@@ -303,11 +292,11 @@ int __ofono_log_init(const char *program, const char *debug,
 	if (detach == FALSE)
 		option |= LOG_PERROR;
 
-#ifdef __GLIBC__
+#ifdef HAVE_BACKTRACE
 	signal_setup(signal_handler);
 #endif
 
-	openlog(basename(program), option, LOG_DAEMON);
+	openlog(l_basename(program), option, LOG_DAEMON);
 
 	syslog(LOG_INFO, "oFono version %s", VERSION);
 
@@ -320,7 +309,7 @@ void __ofono_log_cleanup(void)
 
 	closelog();
 
-#ifdef __GLIBC__
+#ifdef HAVE_BACKTRACE
 	signal_setup(SIG_DFL);
 #endif
 
